//  The contents of this file are subject to the University of Utah Public
//  License (the "License"); you may not use this file except in compliance
//  with the License.
//  
//  Software distributed under the License is distributed on an "AS IS"
//  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
//  License for the specific language governing rights and limitations under
//  the License.
//  
//  The Original Source Code is SCIRun, released March 12, 2001.
//  
//  The Original Source Code was developed by the University of Utah.
//  Portions created by UNIVERSITY are Copyright (C) 2001, 1994
//  University of Utah. All Rights Reserved.
//  
//    File   : AddX.h
//    Author : Martin Cole
//    Date   : Mon Dec 15 14:08:11 2003

#if !defined(AddX_h)
#define AddX_h

#include <Core/Util/TypeDescription.h>
#include <Core/Util/DynamicLoader.h>
#include <Core/Datatypes/Field.h>

#include <iostream>

namespace Workshop03 {

using namespace SCIRun;
using namespace std;


class FieldAddAlgo : public DynamicAlgoBase
{
public:
  virtual void execute(FieldHandle src, FieldHandle &fout_handle, 
		       double x) = 0;

  //! support the dynamically compiled algorithm concept
  static CompileInfoHandle get_compile_info(const TypeDescription *);
};

template <class FLD>
class FieldAddAlgoT : public FieldAddAlgo
{
public:
  //! virtual interface. 
  virtual void execute(FieldHandle src, FieldHandle &fout_handle, double x);
};

template <class FLD>
void 
FieldAddAlgoT<FLD>::execute(FieldHandle in_fld, FieldHandle &fout_handle, 
			    double x)
{
  fout_handle = in_fld;
  fout_handle.detach();
  FLD *fld = dynamic_cast<FLD *>(in_fld.get_rep());
  FLD *fout = dynamic_cast<FLD *>(fout_handle.get_rep());
  typename FLD::mesh_handle_type mesh = fld->get_typed_mesh();

  mesh->synchronize(Mesh::NODES_E);

  typename FLD::mesh_type::Node::iterator niter;  mesh->begin(niter);  
  typename FLD::mesh_type::Node::iterator niter_end;  mesh->end(niter_end);  
  while (niter != niter_end) {
    if (fld->data_at() != Field::NODE) {
      return; 
    }
    typename FLD::value_type tmp;
    fld->value(tmp, *niter);
    
    // will only work on scalar fields....
    fout->set_value(tmp + x, *niter);
    ++niter;
  }

}



} // End namespace Workshop03

#endif // AddX_h
